#cython: language_level=3


from ..column cimport register_column_class

from ...view.view cimport View

from obitools3.utils cimport tobytes, \
                             obi_errno_to_exception

from ...capi.obiview cimport obi_get_bool_with_elt_name_and_col_p_in_view, \
                             obi_get_bool_with_elt_idx_and_col_p_in_view, \
                             obi_set_bool_with_elt_name_and_col_p_in_view, \
                             obi_set_bool_with_elt_idx_and_col_p_in_view

from ...capi.obitypes     cimport OBI_BOOL, OBIBool_NA, obibool_t

from cpython.bool cimport PyBool_FromLong


cdef class Column_bool(Column):

    @staticmethod
    def new(View    view,
            object  column_name,
            index_t nb_elements_per_line=1,
            object  elements_names=None,
            object  comments=b""):
        return Column.new_column(view, column_name, OBI_BOOL,
                          nb_elements_per_line=nb_elements_per_line, 
                          elements_names=elements_names, 
                          comments=comments)
                

    cpdef object get_line(self, index_t line_nb):
        cdef   obibool_t value
        cdef   object    result
        value = obi_get_bool_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0)
        obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem getting a value from a column")
        if value == OBIBool_NA :
            result = None
        else :
            result = PyBool_FromLong(value)
        return result
    
    
    cpdef set_line(self, index_t line_nb, object value):
        if value is None :
            value = OBIBool_NA
        if obi_set_bool_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0, <obibool_t> value) < 0 :
            obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem setting a value in a column")
 
 
cdef class Column_multi_elts_bool(Column_multi_elts):
 
    cpdef object get_item(self, index_t line_nb, object elt_id) :
        cdef   obibool_t value
        cdef   object    result
        cdef   bytes     elt_name
        if type(elt_id) == int :
            value = obi_get_bool_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id)
        else :
            elt_name = tobytes(elt_id)
            value = obi_get_bool_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name)
        obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem getting a value from a column")
        if value == OBIBool_NA :
            result = None
        else :
            result = PyBool_FromLong(value)
        return result
    
    cpdef object get_line(self, index_t line_nb) :
        cdef   obibool_t value
        cdef   object    value_in_result
        cdef   dict      result
        cdef   index_t   i
        cdef   bint      all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_bool_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, i)
            obi_errno_to_exception(line_nb=line_nb, elt_id=i, error_message="Problem getting a value from a column")
            if value == OBIBool_NA :
                value_in_result = None
            else :
                value_in_result = PyBool_FromLong(value)   
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result
    
    cpdef set_item(self, index_t line_nb, object elt_id, object value) :
        cdef   bytes elt_name
        if value is None :
            value = OBIBool_NA
        if type(elt_id) == int :
            if obi_set_bool_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id, <obibool_t> value) < 0 :
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")
        else :
            elt_name = tobytes(elt_id)
            if obi_set_bool_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name, <obibool_t> value) < 0 :
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")


def register_class() :
    register_column_class(OBI_BOOL, False, Column_bool, bool)
    register_column_class(OBI_BOOL, True, Column_multi_elts_bool, bool)








# cdef class Column_line_bool(Column_line) :
# 
#     cdef update_pointer(self):
#         """
#         Checks if the obicolumn address changed since the last call and update
#         if need the `_column_p` and `_data_view` data structure fields.
#         """
#         cdef OBIDMS_column_p* column_pp
#         column_pp = <OBIDMS_column_p*>self._pointer
#         cdef OBIDMS_column_p column_p = column_pp[0]
#         
#         if column_p != self._column_p:
#             self._column_p  = column_p
#             self._data_view = (<obibool_t*> (column_p.data)) + \
#                               self._index * column_p.header.nb_elements_per_line
#     
#     @staticmethod                      
#     cdef bool obibool_t2bool(obibool_t value):
#         cdef bool      result
#         
#         if value == OBIBool_NA :
#             result = None
#         else :
#             result = PyBool_FromLong(value)
#             
#         return result
#     
#     @staticmethod                      
#     cdef bool2obibool_t(bool value):
#         cdef obibool_t result
#         
#         if value is None:
#             result=OBIBool_NA 
#         else:
#             result= <obibool_t> <int> value
#             
#         return result
#         
# 
#     def __init__(self, Column column, index_t line_nb) :
#         """
#         Creates a new `OBIDMS_column_line_bool`
#         
#             @param column: an OBIDMS_column instance
#             @param line_nb: the line in the column 
#         """
#         
#         Column_line.__init__(self, column, line_nb)
#         self.update_pointer()
# 
#         
#         
#     cpdef bool get_bool_item_by_name(self, bytes element_name) :
#         """
#         Returns the value associated to the name `element_name` of the current line
#         
#             @param element_name: a `bytes` instance containing the name of the element
#             
#             @return: the `bool` value corresponding to the name
#         """
#         cdef char*     cname = element_name
#         cdef obibool_t value 
#         global         obi_errno
#         
#         self.update_pointer()
#         
#         cdef OBIDMS_column_p* column_pp
#         column_pp = <OBIDMS_column_p*>self._pointer
#         cdef OBIDMS_column_p column_p = column_pp[0]
# 
#         value = obi_column_get_obibool_with_elt_name(column_p,
#                                                      self._index, 
#                                                      cname)
#         
#         if obi_errno > 0 :
#             obi_errno = 0
#             raise KeyError("Cannot access to key %s" % bytes2str(element_name))
#                     
#         return Column_line_bool.obibool_t2bool(value)
# 
# 
#     cpdef bool get_bool_item_by_idx(self,index_t index):
#         """
#         Returns the value associated to the name `element_name` of the current line
#         
#             @param index: a `int` instance containing the index of the element
#             
#             @return: the `bool` value corresponding to the name
#         """
#         cdef obibool_t value  # @DuplicatedSignature
#         global obi_errno
# 
#         cdef OBIDMS_column_p* column_pp
#         column_pp = <OBIDMS_column_p*>self._pointer
#         cdef OBIDMS_column_p column_p = column_pp[0]
# 
#         self.update_pointer()
#         
#         value = obi_column_get_obibool_with_elt_idx(column_p,
#                                                     self._index, 
#                                                     index)
#         
#         if obi_errno > 0 :
#             obi_errno = 0
#             raise IndexError("Cannot access to element %d" % index)
#                     
#         return Column_line_bool.obibool_t2bool(value)
# 
#         
#     def __getitem__(self, object element_name) :
#         cdef bytes     name
#         cdef int   cindex
#         cdef obibool_t value        
#         cdef type      typearg = type(element_name)
#         cdef bool      result
#         
# 
#         if typearg == int:
#             cindex=element_name
#             if cindex < 0:
#                 cindex = self._len - cindex
#             result=self.get_bool_item_by_idx(cindex)
#         elif typearg == bytes:
#             result=self.get_bool_item_by_name(element_name)
#         elif typearg == str:
#             name = str2bytes(element_name)
#             result=self.get_bool_item_by_name(name)
#         
#         return result
#     
#     cpdef set_bool_item_by_name(self,bytes element_name,bool value):
#         """
#         Sets the value associated to the name `element_name` of the current line
#         
#             @param element_name: a `bytes` instance containing the name of the element
#             @param value: a `bool` instance of the new value
#             
#             @return: the `bool` value corresponding to the name
#         """
#         cdef char*     cname = element_name
#         cdef obibool_t cvalue 
#         
#         self.update_pointer()
#         cvalue = OBIDMS_column_line_bool.bool2obibool_t(value)
# 
#         if ( obi_column_set_obibool_with_elt_name((<OBIDMS_column_p*>self._pointer)[0], 
#                                                   self._index, 
#                                                   cname,
#                                                   cvalue) < 0 ):
#             raise KeyError("Cannot access to key %s" % bytes2str(element_name))
#                     
#     cpdef set_bool_item_by_idx(self,index_t index,bool value):
#         """
#         Sets the value associated to the name `element_name` of the current line
#         
#             @param index: a `int` instance containing the index of the element
#             @param value: a `bool` instance of the new value
#             
#             @return: the `bool` value corresponding to the name
#         """
#         cdef obibool_t cvalue  # @DuplicatedSignature
#         
#         self.update_pointer()
#         cvalue = OBIDMS_column_line_bool.bool2obibool_t(value)
# 
#         if ( obi_column_set_obibool_with_elt_idx((<OBIDMS_column_p*>self._pointer)[0],
#                                                  self._index, 
#                                                  index,
#                                                  cvalue) < 0 ):
#             raise IndexError("Cannot access to item index %d" % index)
#                     
# 
#              
#     def __setitem__(self, object element_name, object value):
#         cdef bytes     name
#         cdef int   cindex
#         cdef type      typearg = type(element_name)
#         cdef bool      result
#         
# 
#         if typearg == int:
#             cindex=element_name
#             if cindex < 0:
#                 cindex = self._len - cindex
#             self.set_bool_item_by_idx(cindex,value)
#         elif typearg == bytes:
#             self.set_bool_item_by_name(element_name,value)
#         elif typearg == str:
#             name = str2bytes(element_name)
#             self.set_bool_item_by_name(name,value)
# 
#     def __repr__(self) :
#         return str(self._column.get_line(self._index))
#     
#     def __len__(self):
#         return self._len
